/* This file is automatically generated.  DO NOT EDIT! */
extern ld_emulation_xfer_type ld_elf_i386_msdos_mz_emulation;
extern ld_emulation_xfer_type ld_elf_i386_emulation;
extern ld_emulation_xfer_type ld_i386elks_emulation;
extern ld_emulation_xfer_type ld_i386msdos_emulation;

#define EMULATION_LIST \
  &ld_elf_i386_msdos_mz_emulation, \
  &ld_elf_i386_emulation, \
  &ld_i386elks_emulation, \
  &ld_i386msdos_emulation, \
  0
